#include "sq.h"

/* NOTE: This file is obsolete since image version 1.18 */
#if (SQ_IMAGE_VERSION_MAJOR == 1) && (SQ_IMAGE_VERSION_MINOR < 18)

/***
	Note: There is a slight asymmetry in error handling here. readImageFile is called
	only from C code at startup time. If it encounters problems, it reports an error
	and quits. If it returns, it is assumed to have succeeded. In contrast, writeImageFile
	is called from a primitive. If it fails, it prints an error, sets successFlag to
	false, and continues. This allows the user to correct the problem and try again.

***/

/*** imported variables (typically initialized by readImageFile, saved by writeImageFile) ***/

extern int endOfMemory, lastHash, memoryLimit, specialObjectsOop, successFlag;
extern unsigned char *memory;
extern int savedWindowSize;
extern char imageName[];

#define SEEK_SET	0
#define SEEK_REL	1
#define SEEK_END	2

/*** local functions ***/
int nextWord(FILE *f);
int nextWordPut(FILE *f, int word);

int nextWord(FILE *f) {
	int result, count;

	count = fread(&result, sizeof(int), 1, f);
	if (count != 1) {
		error("Read failed in nextWord");
	}
	return result;
}

int nextWordPut(FILE *f, int word) {
	int count;

	count = fwrite(&word, sizeof(int), 1, f);
	if (count != 1) {
		printf("Write failed in nextWordPut\n");
		return success(false);
	}
}

/*** image file read/write ***/

int readImageFile(char *imageFilename) {
	FILE	*f;
	int		version, headerSize, dataSize, oldBaseAddr, bytesToShift;
	int		availableMemory, minimumMemory, err, count;
	int		seekAdjust= 0;

	f = fopen(imageFilename, "rb");
	if (f == NULL) {
		printf("Could not open the Squeak image file '%s'\n", imageFilename);
		exit(1);
	}

	version				= nextWord(f);

	if (version != CURRENT_VERSION) {
	  /* perhaps there's a resource fork to ignore? */
	  seekAdjust= 512;
	  if (fseek(f, seekAdjust, SEEK_SET))
	    error("Seek failed; bad image file, perhaps?");
	  version= nextWord(f);
	  if (version != CURRENT_VERSION) {
	    printf("This interpreter (vers. %d) cannot read image file (vers. %d)\n",
		   CURRENT_VERSION, version);
	    error("Aborting...");
	  }
	}

	headerSize			= nextWord(f);
	dataSize			= nextWord(f);
	oldBaseAddr			= nextWord(f);
	specialObjectsOop		= nextWord(f);
	lastHash			= nextWord(f);
	savedWindowSize		= nextWord(f);
printf ("Header size is %d\n", headerSize);
printf ("Data size is %d\n", dataSize);
printf ("Old base addr is %d\n", oldBaseAddr);
printf ("specialObjectsOop is %d\n", specialObjectsOop);
printf ("Saved window size is %d\n", savedWindowSize);

	if (lastHash == 0) {
		/* lastHash wasn't stored (e.g. by the cloner); use 999 as the seed */
		lastHash = 999;
	}

	/* compute memory requirements and availability */
	minimumMemory = dataSize + 20000;
	availableMemory = dataSize + 4000000;	/* 4Mb of available image space */

	if (availableMemory < minimumMemory) {
		error("Could not allocate memory for the heap");
	}

	/* allocate a contiguous block of memory for the Squeak heap */
	memory = (unsigned char *)calloc(availableMemory, 1);
	if (memory == NULL) {
		error("Failed to allocate memory for the heap");
	}
	memoryLimit = (((int) memory) + availableMemory) - 24;  /* decrease memoryLimit a tad for safety */
	endOfMemory = ((int) memory) + dataSize;

	/* position file after the header */
	err = fseek(f, headerSize+seekAdjust, SEEK_SET);
	if (err) {
		error("Seek failed; bad image file, perhaps?");
	}

	/* read in the image */
	count = fread(memory, sizeof(unsigned char), dataSize, f);
	if (count != dataSize) {
		error("Read failed or premature end of image file");
	}

	fclose(f);

	/* compute difference between old and new memory base addresses */
	bytesToShift = ((int) memory) - oldBaseAddr;

	initializeInterpreter(bytesToShift);  /* adjusts all oops to new location */

	return dataSize;
}

int writeImageFile(int imageBytes) {
	int		headerSize = 64;  /* header size in bytes */
	int		err, count;
	FILE	*f;

	successFlag = true;

	f = fopen(imageName, "w");
	if (f == NULL) {
		printf("Could not open image file for writing: %s\n", imageName);
		return success(false);
	}

	nextWordPut(f, CURRENT_VERSION);
	nextWordPut(f, headerSize);
	nextWordPut(f, imageBytes);
	nextWordPut(f, (int) memory);
	nextWordPut(f, specialObjectsOop);
	nextWordPut(f, lastHash);
	nextWordPut(f, ioScreenSize());
	if (!successFlag) {
		return success(false);
	}

	/* position file after the header */
	err = fseek(f, headerSize, SEEK_SET);
	if (err) {
		printf("Seek failed in writeImageFile\n");
		return success(false);
	}

	/* write out the image */
	count = fwrite(memory, sizeof(unsigned char), imageBytes, f);
	if (count != imageBytes) {
		printf("Writing image file failed; disk full, perhaps?\n");
		return success(false);
	}

	err = fclose(f);
	if (err) {
		printf("Closing image file failed; disk full, perhaps?\n");
		return success(false);
	}
	dir_SetMacFileTypeAndCreator(imageName, strlen(imageName), "STim", "FAST");
}


#endif /* (SQ_IMAGE_VERSION_MAJOR == 1) && (SQ_IMAGE_VERSION_MINOR < 18) */


int swapBytes(int word)
{
  int conv;
  unsigned char *cp= (unsigned char *)&conv, *wp= (unsigned char *)&word;

  cp[0]= wp[3];
  cp[1]= wp[2];
  cp[2]= wp[1];
  cp[3]= wp[0];
  return conv;
}
