/****************************************************************************
*   PROJECT: Squeak port for Win32 (NT / Win95)
*   FILE:    sqWin32Net.c
*   CONTENT: Networking in Squeak
*
*   AUTHOR:  Andreas Raab (ar)
*   ADDRESS: University of Magdeburg, Germany
*   EMAIL:   raab@isg.cs.uni-magdeburg.de
*   RCSID:   $Id: sqWin32Net.c,v 1.8 1997/10/14 18:03:00 raab Exp $
*
*   NOTES:   1) For now, only TCP is supported.
*            2) The async notifications require message processing.
*               Therefore, each function has a preceeding call to ioProcessEvents
*
*****************************************************************************/
#ifndef NO_NETWORKING

#include "sq.h"
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <errno.h>


/*** Socket TYpe Constants ***/
#define TCPSocketType 0
#define UDPSocketType 1

/*** Resolver Status Constants ***/
#define RESOLVER_UNINITIALIZED 0
#define RESOLVER_SUCCESS  1
#define RESOLVER_BUSY   2
#define RESOLVER_ERROR   3

/*** TCP Socket Status Constants ***/
#define Unconnected    0
#define WaitingForConnection 1
#define Connected    2
#define OtherEndClosed   3
#define ThisEndClosed   4

static char RCSID[]="$Id: sqWin32Net.c,v 1.8 1997/10/14 18:03:00 raab Exp $";

static int resolverSemaphoreIndex = 0;
static int thisNetSession = 0;
static int zero = 0;
static int one = 1;

#define h_errno 3
#define AF_INET 2
#define MAXHOSTNAMELEN 256
static char localHostName[MAXHOSTNAMELEN];
static long localHostAddress;
#define MAXGETHOSTSTRUCT 2
typedef struct privateSocketStruct {int bob;} privateSocketStruct;


static privateSocketStruct *firstSocket = NULL;
/* Additional flags for sockState which will be received by async notification:
   SOCK_DATA_SENT      - all pending data has been sent
   SOCK_DATA_AVAILABLE - data is available for this connection
*/
#define SOCK_DATA_SENT      0x00010000
#define SOCK_DATA_AVAILABLE 0x00020000

#define SOCKET(s) (((privateSocketStruct*)s->privateSocketPtr)->s)
#define SOCKETSTATE(s) (((privateSocketStruct*)s->privateSocketPtr)->sockState)
#define SOCKETERROR(s) (((privateSocketStruct*)s->privateSocketPtr)->sockError)

#define WM_NET_MESSAGE (WM_USER+1)
#define WM_RESOLVER_MESSAGE (WM_USER+2)

int sqNetMessageHook(int message,int wParam,int lParam);
static int nextMessageHook = NULL;
static int hookInstalled = 0;

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqNetworkInit(int resolverSemaIndex)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
void sqNetworkShutdown(void)
{}

/*** Squeak Generic Socket Functions ***/

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
static int SocketValid(SocketPtr s) {}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
void sqSocketAbortConnection(SocketPtr s){}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
void sqSocketCloseConnection(SocketPtr s){}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqSocketConnectionStatus(SocketPtr s){}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
void sqSocketConnectToPort(SocketPtr s, int addr, int port){}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
void sqSocketListenOnPort(SocketPtr s, int port){}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
void sqSocketCreateNetTypeSocketTypeRecvBytesSendBytesSemaID(
            SocketPtr s, int netType, int socketType,
            int recvBufSize, int sendBufSize, int semaIndex)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
/* Release the resources associated with this socket. 
   If a connection is open, it is aborted.*/
void sqSocketDestroy(SocketPtr s)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqSocketError(SocketPtr s)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqSocketLocalAddress(SocketPtr s)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqSocketLocalPort(SocketPtr s)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqSocketReceiveDataAvailable(SocketPtr s)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqSocketReceiveDataBufCount(SocketPtr s, int buf, int bufSize)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqSocketRemoteAddress(SocketPtr s)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqSocketRemotePort(SocketPtr s)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqSocketSendDataBufCount(SocketPtr s, int buf, int bufSize)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqSocketSendDone(SocketPtr s)
{}



/*****************************************************************************
*** Resolver Functions ***
*****************************************************************************/

static char lastName[MAXHOSTNAMELEN+1];
static int lastAddr;

static int lastError;
static char hostentBuffer[MAXGETHOSTSTRUCT];

/*****************************************************************************
  Convenience functions. We may use them later.
*****************************************************************************/
/*static char *
nameOf(int netAddress)
{ int nAddr;
  struct hostent *he;

  lastError = 0;
  nAddr = htonl(netAddress);
  he = gethostbyaddr((char*)&nAddr,sizeof(nAddr),AF_INET);
  if(he) return he->h_name;
  lastError = h_errno;
  return "";
}

static int
addressOf(char *hostName)
{ struct hostent *he;

  lastError = 0;
  he = gethostbyname(hostName);
  if(he) return ntohl(*(long*)(he->h_addr_list[0]));
  lastError = h_errno;
  return 0;
}
*/
/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
void sqResolverAbort(void)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
void sqResolverAddrLookupResult(char *nameForAddress, int nameSize)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqResolverAddrLookupResultSize(void)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqResolverError(void)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqResolverLocalAddress(void)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqResolverNameLookupResult(void)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
void sqResolverStartAddrLookup(int address)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
void sqResolverStartNameLookup(char *hostName, int nameSize)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqResolverStatus(void)
{}

/*****************************************************************************
 WinSocket specific async notifications
*****************************************************************************/

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
void processAsyncResolverEvent(int h, int lParam)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
void processAsyncSocketEvent(int s, int event, int errorCode)
{}

/*****************************************************************************
  Tested - Ok.
*****************************************************************************/
int sqNetMessageHook(int message,int wParam,int lParam)
{}

#endif /* ifndef NO_NETWORKING */
